/**
 * 
 */
package org.processmining.plugins.log.visualization;


/**
 * @author aadrians
 * Jul 28, 2012
 *
 */
public abstract class LogPerformanceMatrixProvider {
	public enum Metric{ AVG, STDDEV, MIN, MAX };

	protected String classLbl[] = null;
	protected double[] data = null; // min, max, average, stddev
	protected int[] occ = null;

	protected void updateMandSVal(int i, int j, double[] data, int[] occ, long val, int length) {
		int loc = (length * j) + i;
		int pos = 4 * loc;
		if (Double.isNaN(data[pos])) {
			data[pos] = val; // min
			data[pos + 1] = val; // max
			data[pos + 2] = val; // mval
			data[pos + 3] = 0; // sval
			occ[loc] = 1; // occurrence
		} else {
			occ[loc]++;
			if (Double.compare(data[pos], val) > 0){
				data[pos] = val; // min				
			}
			if (Double.compare(data[pos + 1], val) < 0){
				data[pos + 1] = val; // max
			}
			double oldMVal = data[pos + 2];
			data[pos + 2] += (val - oldMVal) / occ[loc];
			data[pos + 3] += ((val - oldMVal) * (val - data[pos + 2]));
		}
	}
	
	public int getNumClasses() {
		return classLbl.length;
	}

	public double getTimeBetween(int i, int j, Metric metric) {
		int loc = 4 * ((i * getNumClasses()) + j);
		switch (metric){
			case MIN:
				return (this.data[loc]);
			case MAX:
				return (this.data[loc + 1]);
			case AVG:
				return (this.data[loc + 2]);
			case STDDEV:
				return (this.data[loc + 3]);
		}
		return Double.NaN;
	}

	public String[] getClassesLabel() {
		return classLbl;
	}

	public double getFrequency(int i, int j) {
		return (this.occ[(i * getNumClasses()) + j]);	
	}
}
