/**
 * 
 */
package org.processmining.plugins.pnalignanalysis.export;

import info.clearthought.layout.TableLayout;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JPanel;

import com.fluxicon.slickerbox.components.NiceDoubleSlider;
import com.fluxicon.slickerbox.components.NiceSlider.Orientation;
import com.fluxicon.slickerbox.factory.SlickerFactory;

/**
 * @author aadrians Jan 20, 2012
 * 
 */
public class LogCreatorPanel extends JPanel {
	private static final long serialVersionUID = 2523245998920248980L;

	private JCheckBox toNewEvClass;
	private JCheckBox moveModelBox;
	private JCheckBox moveLogBox;
	private JCheckBox unmappedBox;
	private JCheckBox fitnessFilter;
	private JComboBox algorithm;

	private NiceDoubleSlider sliderMax;
	private NiceDoubleSlider sliderMin;

	public LogCreatorPanel(ILogCreatorAlgorithm[] algorithms) {
		super();
		//Factory to create ProM swing components
		SlickerFactory factory = SlickerFactory.instance();

		double[][] size = new double[][] { { 10.0, 150.0, TableLayout.FILL }, { 30.0 } };
		int rowCounter = 1;

		TableLayout layout = new TableLayout(size);
		setLayout(layout);

		// get algorithms
		algorithm = factory.createComboBox(algorithms);

		//Setting the "table"
		moveModelBox = factory.createCheckBox("Create new events for move on model (based on their event class mapping)", false);
		moveLogBox = factory.createCheckBox("Include \"move on log\" events", true);

		toNewEvClass = factory.createCheckBox("Create new event classes based on type of movements", false);
		ILogCreatorAlgorithm sel = (ILogCreatorAlgorithm) algorithm.getSelectedItem();
		toNewEvClass.setEnabled(sel.isSupportCreateNewEventClassBasedOnMovements());
		
		unmappedBox = factory.createCheckBox("Create new events for move on models with unmapped (invisible) transitions",
				false);
		unmappedBox.setEnabled(sel.isSupportCreateNewEventClassBasedOnMovements()
				&& sel.isSupportCreateNewEventClassForUnmappedTransitions());
		
		fitnessFilter = factory.createCheckBox("Only consider alignments with fitness range", false);

		sliderMin = factory.createNiceDoubleSlider("Min. fitness", 0.0, 1.0, 0.0, Orientation.HORIZONTAL);
		sliderMin.setEnabled(false);
		sliderMax = factory.createNiceDoubleSlider("Max. fitness", 0.0, 1.0, 1.0, Orientation.HORIZONTAL);
		sliderMax.setEnabled(false);

		fitnessFilter.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				sliderMin.setEnabled(fitnessFilter.isSelected());
				sliderMax.setEnabled(fitnessFilter.isSelected());
			}
		});

		algorithm.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent arg0) {
				ILogCreatorAlgorithm sel = (ILogCreatorAlgorithm) algorithm.getSelectedItem();

				if (toNewEvClass.isSelected()) {
					toNewEvClass.setSelected(sel.isSupportCreateNewEventClassBasedOnMovements());
				}
				toNewEvClass.setEnabled(sel.isSupportCreateNewEventClassBasedOnMovements());

				if (unmappedBox.isSelected()) {
					unmappedBox.setSelected(sel.isSupportCreateNewEventClassBasedOnMovements()
							&& sel.isSupportCreateNewEventClassForUnmappedTransitions());
				}
				
				unmappedBox.setEnabled(sel.isSupportCreateNewEventClassBasedOnMovements()
						&& sel.isSupportCreateNewEventClassForUnmappedTransitions());

			}
		});

		// add components to table
		add(factory.createLabel("Select Algorithm"), "0,0,1,0");
		add(algorithm, "2,0");

		layout.insertRow(rowCounter, 30.0);
		rowCounter++;

		layout.insertRow(rowCounter, 30.0);
		add(factory.createLabel("Common Settings"), "0," + rowCounter + ",1," + rowCounter);

		add(moveModelBox, "2," + rowCounter);
		rowCounter++;

		layout.insertRow(rowCounter, 30.0);
		add(moveLogBox, "2," + rowCounter);
		rowCounter++;

		layout.insertRow(rowCounter, 30.0);
		add(fitnessFilter, "2," + rowCounter);
		rowCounter++;

		layout.insertRow(rowCounter, 30.0);
		add(sliderMin, "2," + rowCounter);
		rowCounter++;

		layout.insertRow(rowCounter, 30.0);
		add(sliderMax, "2," + rowCounter);
		rowCounter++;

		layout.insertRow(rowCounter, 30.0);
		rowCounter++;
		
		layout.insertRow(rowCounter, 30.0);
		add(factory.createLabel("Extras (if supported)"), "0," + rowCounter + ",1," + rowCounter);

		add(toNewEvClass, "2," + rowCounter);
		rowCounter++;

		layout.insertRow(rowCounter, 30.0);
		add(unmappedBox, "2," + rowCounter);
		rowCounter++;

	}

	public boolean isCreateEventClassesBasedOnMovements() {
		return toNewEvClass.isSelected();
	}

	public boolean isCreateEventForMoveModel() {
		return moveModelBox.isSelected();
	}

	public boolean isCreateEventForMoveLog() {
		return moveLogBox.isSelected();
	}

	public boolean isCreateEventForUnmappedTransition() {
		return unmappedBox.isSelected();
	}

	public double getMinFitnessValue() {
		return fitnessFilter.isSelected() ? sliderMin.getValue() : -1 * (Double.MAX_VALUE);
	}

	public double getMaxFitnessValue() {
		return fitnessFilter.isSelected() ? sliderMax.getValue() : Double.MAX_VALUE;
	}

	public ILogCreatorAlgorithm getAlgorithm() {
		return (ILogCreatorAlgorithm) algorithm.getSelectedItem();
	}
}
