/**
 * 
 */
package org.processmining.plugins.pnalignanalysis.export;

import gnu.trove.map.TIntObjectMap;
import gnu.trove.map.TObjectIntMap;
import gnu.trove.map.hash.TIntObjectHashMap;
import gnu.trove.map.hash.TObjectIntHashMap;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.Iterator;
import java.util.List;

import nl.tue.astar.util.ShortShortMultiset;

import org.deckfour.xes.classification.XEventClass;
import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.extension.std.XTimeExtension;
import org.deckfour.xes.factory.XFactory;
import org.deckfour.xes.factory.XFactoryRegistry;
import org.deckfour.xes.model.XAttribute;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.KeepInProMCache;
import org.processmining.models.connections.petrinets.PNRepResultAllRequiredParamConnection;
import org.processmining.models.connections.petrinets.behavioral.InitialMarkingConnection;
import org.processmining.models.graphbased.directed.petrinet.PetrinetEdge;
import org.processmining.models.graphbased.directed.petrinet.PetrinetGraph;
import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.models.graphbased.directed.petrinet.elements.Arc;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.ResetArc;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.models.semantics.petrinet.Marking;
import org.processmining.plugins.connectionfactories.logpetrinet.TransEvClassMapping;
import org.processmining.plugins.petrinet.replayresult.PNRepResult;
import org.processmining.plugins.petrinet.replayresult.StepTypes;
import org.processmining.plugins.replayer.replayresult.SyncReplayResult;

/**
 * This creator adds timestamps assumes move on model as immediately firing
 * transitions
 * 
 * @author aadrians Nov 15, 2012
 * 
 */
@KeepInProMCache
@LogCreatorFromAlignment
public class ReplayLogCreator implements ILogCreatorAlgorithm {

	public String toString() {
		return "Insert timestamps based on alignment replay";
	}

	public XLog extractLog(PetrinetGraph net, Marking initMarking, XLog log, TransEvClassMapping mapping,
			PNRepResult logReplayResult, LogCreatorParam param) {

		/**
		 * create encoded net and marking
		 */
		// utility: encode the net
		Transition[] idx2Trans;
		TObjectIntMap<Transition> trans2Idx;

		Place[] idx2Place;
		TObjectIntMap<Place> place2Idx;

		TIntObjectHashMap<short[]> encodedTrans2Pred;
		TIntObjectHashMap<short[]> encodedTrans2Succ;

		// encode net
		// init transitions
		List<Transition> transitions = new ArrayList<Transition>(net.getTransitions());
		int transSize = transitions.size();
		idx2Trans = transitions.toArray(new Transition[transSize]);
		trans2Idx = new TObjectIntHashMap<Transition>(transSize);
		for (int i = 0; i < idx2Trans.length; i++) {
			trans2Idx.put(idx2Trans[i], i);
		}

		// init places
		List<Place> places = new ArrayList<Place>(net.getPlaces());
		int placeSize = places.size();
		idx2Place = places.toArray(new Place[placeSize]);
		place2Idx = new TObjectIntHashMap<Place>(placeSize);
		for (int i = 0; i < idx2Place.length; i++) {
			place2Idx.put(idx2Place[i], i);
		}

		// init input and output place
		encodedTrans2Pred = new TIntObjectHashMap<short[]>(idx2Trans.length);
		encodedTrans2Succ = new TIntObjectHashMap<short[]>(idx2Trans.length);
		for (int i = 0; i < transSize; i++) {

			Collection<PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode>> inEdges = net
					.getInEdges(idx2Trans[i]);
			if (inEdges != null) {
				short[] newIn = new short[placeSize];
				for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> edge : inEdges) {
					int sourceEnc = place2Idx.get(edge.getSource());
					if (edge instanceof ResetArc) {
						newIn[sourceEnc] = (short) (-newIn[sourceEnc] - 1);
					} else if (edge instanceof Arc) {
						if (newIn[sourceEnc] < 0) {
							newIn[sourceEnc] = (short) -net.getArc(edge.getSource(), edge.getTarget()).getWeight();
						} else {
							newIn[sourceEnc] = (short) net.getArc(edge.getSource(), edge.getTarget()).getWeight();
						}
					}
				}
				;
				encodedTrans2Pred.put(i, newIn);
			}

			Collection<PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode>> outEdges = net
					.getOutEdges(idx2Trans[i]);
			if (outEdges != null) {
				short[] newOut = new short[placeSize];
				for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> edge : outEdges) {
					newOut[place2Idx.get(edge.getTarget())] = (short) net.getArc(edge.getSource(), edge.getTarget())
							.getWeight();
				}
				;
				encodedTrans2Succ.put(i, newOut);
			}
		}

		/**
		 * Next step: replay and create log
		 */

		// utilities
		// create a log based on alignment
		XConceptExtension conceptExt = XConceptExtension.instance();
		XFactory factory = XFactoryRegistry.instance().currentDefault();

		short[] encInitMarking = new short[idx2Place.length];
		for (Place p : initMarking.baseSet()) {
			encInitMarking[place2Idx.get(p)] = initMarking.occurrences(p).shortValue();
		}

		// initialize timestamp
		TIntObjectMap<List<Long>> timedPlaces = new TIntObjectHashMap<List<Long>>(idx2Place.length);
		XTimeExtension tx = XTimeExtension.instance();

		// result
		XLog outputLog = factory.createLog();
		outputLog.getClassifiers().addAll(log.getClassifiers()); // copy previous classifiers

		for (SyncReplayResult res : logReplayResult) {
			// get the index where first timestamp exists
			int firstTimestampIdx = 0;
			Iterator<StepTypes> tempIdxIt = res.getStepTypes().iterator();
			while (tempIdxIt.hasNext()) {
				StepTypes step = tempIdxIt.next();
				if (step.equals(StepTypes.LMGOOD)) {
					break;
				} else if (step.equals(StepTypes.L)) {
					firstTimestampIdx++;
				}
			}
			if (firstTimestampIdx > res.getStepTypes().size()) {
				// no move sync, just put random
				firstTimestampIdx = -1;
			}

			for (int caseID : res.getTraceIndex()) {
				XTrace oldTrace = log.get(caseID);
				Iterator<XEvent> it = oldTrace.iterator();
				Iterator<Object> nit = res.getNodeInstance().iterator();
				Iterator<StepTypes> stit = res.getStepTypes().iterator();

				// search for init time
				Long firstTimestamp = null;
				if (firstTimestampIdx >= 0) {
					firstTimestamp = tx.extractTimestamp(oldTrace.get(firstTimestampIdx)).getTime();
				} else {
					firstTimestamp = Long.MIN_VALUE;
				}
				for (int i = 0; i < encInitMarking.length; i++) {
					if (encInitMarking[i] > 0) {
						ArrayList<Long> arrList = new ArrayList<Long>(1);
						arrList.add(firstTimestamp);
						timedPlaces.put(i, arrList);

						for (int j = 1; j < encInitMarking[i]; j++) {
							arrList.add(firstTimestamp);
						}
					}
				}

				// create new trace
				XTrace newTrace = factory.createTrace(oldTrace.getAttributes());
				while (stit.hasNext()) {
					switch (stit.next()) {
						case L :
							if (param.isCreateEventsForMoveLog()) {
								newTrace.add(factory.createEvent(it.next().getAttributes()));
							} else {
								it.next();
							}
							nit.next();
							break;
						case LMGOOD :
							// copy the existing event
							XEvent event = it.next();
							newTrace.add(factory.createEvent(event.getAttributes()));
							int transIdx = trans2Idx.get(nit.next());
							fireTransition(timedPlaces, encodedTrans2Pred.get(transIdx),
									encodedTrans2Succ.get(transIdx), tx.extractTimestamp(event).getTime());
							break;
						case MINVI :
							// do nothing
							nit.next();
							break;
						case MREAL :
							// create new event, if it is mapped to non dummy event class
							Transition t = (Transition) nit.next();
							int encodedTrans = trans2Idx.get(t);
							XEventClass ec = mapping.get(t);
							if (!ec.equals(mapping.getDummyEventClass())) {
								XEvent newEvent = factory.createEvent();
								String[] attValues = ec.getId().split("\\+");
								String[] attKeys = mapping.getEventClassifier().getDefiningAttributeKeys();
								for (int i = 0; i < attKeys.length; i++) {
									XAttribute att = factory.createAttributeLiteral(attKeys[i], attValues[i],
											conceptExt);
									newEvent.getAttributes().put(attKeys[i], att);
								}

								// add timestamp attribute
								// get the latest taken timestamp
								Long time = fireTransition(timedPlaces, encodedTrans2Pred.get(encodedTrans),
										encodedTrans2Succ.get(encodedTrans), null);
								XAttribute att = factory.createAttributeTimestamp("time:timestamp", time, conceptExt);
								newEvent.getAttributes().put("time:timestamp", att);

								// find a position in the trace
								Date currTimestamp = tx.extractTimestamp(newEvent);
								int idx = newTrace.size() - 1;
								while ((idx >= 0) && (tx.extractTimestamp(newTrace.get(idx)).after(currTimestamp))) {
									idx--;
								}
								newTrace.add(idx + 1, newEvent);
							}
							break;
						default :
							it.next();
							nit.next();
							// do nothing for other type of movements
							break;
					}
				}
				outputLog.add(newTrace);

			}
		}

		return outputLog;
	}

	/**
	 * Return the latest time taken in predecessors
	 * 
	 * @param timedPlaces
	 * @param predecessors
	 * @param successors
	 * @param firingTime
	 *            null if the time is unknown because of move model
	 * @return
	 */
	private Long fireTransition(TIntObjectMap<List<Long>> timedPlaces, short[] predecessors, short[] successors,
			Long firingTime) {
		Long latestTime = Long.MIN_VALUE;
		for (int i = 0; i < predecessors.length; i++) {
			if (predecessors[i] > 0) {
				Iterator<Long> it = timedPlaces.get(i).iterator();
				for (int j = 0; j < predecessors[i]; j++) {
					Long predTime = it.next();
					it.remove();
					latestTime = predTime.compareTo(latestTime) > 0 ? predTime : latestTime;
				}
			} else if (predecessors[i] < 0) {
				// reset arcs
				List<Long> list = timedPlaces.get(predecessors[i]);
				if (list != null) {
					if (firingTime == null) {
						list.clear();
					} else {
						Iterator<Long> it = list.iterator();
						while (it.hasNext()) {
							Long predTime = it.next();
							it.remove();
							latestTime = predTime.compareTo(latestTime) > 0 ? predTime : latestTime;
						}
					}
				}
			}
		}

		// add tokens successors
		for (int i = 0; i < successors.length; i++) {
			if (successors[i] > 0) {
				List<Long> list = timedPlaces.get(i);
				if (list == null) {
					list = new ArrayList<Long>(1);
					timedPlaces.put(i, list);
				}
				if (firingTime != null) {
					for (int j = 0; j < successors[i]; j++) {
						list.add(firingTime);
					}
				} else {
					for (int j = 0; j < successors[i]; j++) {
						list.add(latestTime); // immediately fired
					}
				}
			}
		}
		return latestTime;
	}

	public XLog extractLog(PluginContext context, PNRepResult logReplayResult, XLog log, LogCreatorParam param) {
		assert (context != null);
		try {
			PNRepResultAllRequiredParamConnection conn = context.getConnectionManager().getFirstConnection(
					PNRepResultAllRequiredParamConnection.class, context, log, logReplayResult);
			PetrinetGraph net = conn.getObjectWithRole(PNRepResultAllRequiredParamConnection.PN);
			TransEvClassMapping mapping = (TransEvClassMapping) conn
					.getObjectWithRole(PNRepResultAllRequiredParamConnection.TRANS2EVCLASSMAPPING);

			// get initial marking
			InitialMarkingConnection connInitMarking = context.getConnectionManager().getFirstConnection(
					InitialMarkingConnection.class, context, net);
			Marking initMarking = (Marking) connInitMarking.getObjectWithRole(InitialMarkingConnection.MARKING);

			return extractLog(net, initMarking, log, mapping, logReplayResult, param);
		} catch (ConnectionCannotBeObtained exc) {
			return null;
		}
	}

	protected ShortShortMultiset constructEncInitMarking(Marking mNewNet, Place[] idx2Place,
			TObjectIntMap<Place> place2Idx) {
		ShortShortMultiset res = new ShortShortMultiset((short) idx2Place.length);
		for (Place p : mNewNet.baseSet()) {
			res.adjustValue((short) place2Idx.get(p), mNewNet.occurrences(p).shortValue());
		}
		return res;
	}

	public boolean isSupportCreateNewEventClassBasedOnMovements() {
		return false;
	}

	public boolean isSupportCreateNewEventClassForUnmappedTransitions() {
		return false;
	}

}
