/**
 * 
 */
package org.processmining.plugins.pnalignanalysis.filter;

import java.util.Collection;
import java.util.Iterator;

import org.deckfour.xes.classification.XEventClass;
import org.deckfour.xes.classification.XEventClasses;
import org.deckfour.xes.classification.XEventClassifier;
import org.deckfour.xes.info.XLogInfo;
import org.deckfour.xes.info.XLogInfoFactory;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.models.connections.petrinets.EvClassLogPetrinetConnection;
import org.processmining.models.graphbased.directed.petrinet.PetrinetGraph;
import org.processmining.plugins.connectionfactories.logpetrinet.TransEvClassMapping;

/**
 * @author aadrians May 31, 2012
 * 
 */
@Plugin(name = "Filter out unmapped event classes", parameterLabels = {
		"Petrinet", "Mapping", "Event Logs" }, help = "Filter out unmapped event classes.", userAccessible = true, returnLabels = { }, returnTypes = { })
public class FilterOutUnmappedEvents {

	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "Arya Adriansyah", email = "a.adriansyah@tue.nl", pack = "PNetAlignmentAnalysis")
	@PluginVariant(variantLabel = "From Petri net and Event Log", requiredParameterLabels = { 0, 2 })
	public void filter(final UIPluginContext context, PetrinetGraph net, XLog... log) {
		try {
			// get connection between the net and the first log
			EvClassLogPetrinetConnection conn = context.getConnectionManager().getFirstConnection(
					EvClassLogPetrinetConnection.class, context, net, log[0]);
			filter((TransEvClassMapping) conn.getObjectWithRole(EvClassLogPetrinetConnection.TRANS2EVCLASSMAPPING), log);
		} catch (ConnectionCannotBeObtained e) {
			e.printStackTrace();
		}
		
		// regardless what happened, just return null
		context.getFutureResult(0).cancel(true);
	}
	
	@PluginVariant(variantLabel = "From Petri net and Event Log", requiredParameterLabels = { 1, 2 })
	public void filter(final PluginContext context, TransEvClassMapping mapping, XLog... logs) {
		filter(mapping, logs);
	}
	

	public XLog[] filter(TransEvClassMapping mapping, XLog... logs) {
		// util
		Collection<XEventClass> mapped = mapping.values();
		XEventClassifier classifier = mapping.getEventClassifier();

		for (int i = 0; i < logs.length; i++) {
			// create a log
			XLogInfo info = XLogInfoFactory.createLogInfo(logs[i], classifier);
			XEventClasses ec = info.getEventClasses();

			for (XTrace t : logs[i]) {
				Iterator<XEvent> it = t.iterator();
				while (it.hasNext()) {
					if (!mapped.contains(ec.getClassOf(it.next()))) {
						it.remove();
					}
				}
			}
		}
		
		return logs;
	}

}
