/**
 * 
 */
package org.processmining.plugins.pnalignanalysis.visualization.tracealignment;

import java.util.Set;
import java.util.concurrent.CancellationException;
import java.util.concurrent.ExecutionException;

import javax.swing.JComponent;

import org.deckfour.xes.model.XLog;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.PluginContextID;
import org.processmining.framework.plugin.PluginExecutionResult;
import org.processmining.framework.plugin.PluginParameterBinding;
import org.processmining.framework.plugin.events.Logger;
import org.processmining.plugins.guidetreeminer.GuideTreeMinerInput;
import org.processmining.plugins.guidetreeminer.tree.GuideTree;
import org.processmining.plugins.guidetreeminer.types.AHCJoinType;
import org.processmining.plugins.guidetreeminer.types.DistanceMetricType;
import org.processmining.plugins.guidetreeminer.types.GTMFeature;
import org.processmining.plugins.guidetreeminer.types.GTMFeatureType;
import org.processmining.plugins.guidetreeminer.types.LearningAlgorithmType;
import org.processmining.plugins.guidetreeminer.types.SimilarityDistanceMetricType;
import org.processmining.plugins.tracealignmentwithguidetree.TraceAlignmentWithGuideTreeInput;
import org.processmining.plugins.tracealignmentwithguidetree.enumtypes.AlignmentAlgorithm;
import org.processmining.plugins.tracealignmentwithguidetree.enumtypes.ScoringMatrix;
import org.processmining.plugins.tracealignmentwithguidetree.tree.AlignmentTree;
import org.processmining.plugins.tracealignmentwithguidetree.visualization.AlignmentFrame;

/**
 * @author aadrians Jun 2, 2012
 * 
 */
public class TraceAlignmentDelegate {
	public static JComponent createAlignmentTree(final PluginContext context, XLog log,
			ActivityColorMap activityColorMap) throws Exception {
		ChildLogger logger = new ChildLogger();

		final PluginContext child = context.createChildContext("trace-alignment");
		context.getPluginLifeCycleEventListeners().firePluginCreated(child);
		child.getLoggingListeners().add(logger);

		GuideTree guideTree = null;
		GuideTreeMinerInput guideTreeMinerInput = new GuideTreeMinerInput();
		guideTreeMinerInput.setFeatureType(GTMFeatureType.Alphabet);
		guideTreeMinerInput.addFeature(GTMFeature.MRA);
		guideTreeMinerInput.setSimilarityDistanceMetricType(SimilarityDistanceMetricType.Distance);
		guideTreeMinerInput.setDistanceMetricType(DistanceMetricType.Euclidean);
		guideTreeMinerInput.setNominalFeatureCount(false);
		guideTreeMinerInput.setBaseFeatures(false);
		guideTreeMinerInput.setLearningAlgorithmType(LearningAlgorithmType.AHC);
		guideTreeMinerInput.setAhcJoinType(AHCJoinType.MinVariance);
		guideTreeMinerInput.setNumberOfClusters(3);

		Set<PluginParameterBinding> plugins = context.getPluginManager().getPluginsAcceptingOrdered(child.getClass(),
				false, log.getClass(), guideTreeMinerInput.getClass());
		for (PluginParameterBinding pBinding : plugins) {
			if (pBinding.getPlugin().getName().equals("Guide Tree Miner")) {
				PluginExecutionResult guideTreeResult = pBinding.invoke(child, log, guideTreeMinerInput);

				try {
					guideTreeResult.synchronize();
				} catch (CancellationException e) {
					context.getFutureResult(0).cancel(true);
				} catch (ExecutionException e) {
					context.getFutureResult(0).cancel(true);
				} catch (InterruptedException e) {
					context.getFutureResult(0).cancel(true);
				}

				guideTree = guideTreeResult.getResult(0);
				break;
			}
		}

		if (guideTree != null) {
			// call alignment tree
			final PluginContext anotherChild = context.createChildContext("trace-alignment");
			context.getPluginLifeCycleEventListeners().firePluginCreated(anotherChild);
			anotherChild.getLoggingListeners().add(logger);

			TraceAlignmentWithGuideTreeInput traceAlignmentInput = new TraceAlignmentWithGuideTreeInput();
			traceAlignmentInput.setIsDeriveSubstitutionIndelScores(true);
			traceAlignmentInput.setScoringMatrix(ScoringMatrix.LikePreference);
			traceAlignmentInput.setIsIncrementLikeSubstitutionScores(false);
			traceAlignmentInput.setIsScaleIndelScores(false);
			traceAlignmentInput.setAlignmentAlgorithm(AlignmentAlgorithm.ProfileAlignment);

			Set<PluginParameterBinding> plugins2 = context.getPluginManager().getPluginsAcceptingOrdered(
					anotherChild.getClass(), false, guideTree.getClass(), traceAlignmentInput.getClass());

			for (PluginParameterBinding pBinding2 : plugins2) {
				if (pBinding2.getPlugin().getName().equals("Trace Alignment (with Guide Tree)")) {
					PluginExecutionResult traceAlignmentResult = pBinding2.invoke(anotherChild, guideTree,
							traceAlignmentInput);

					try {
						traceAlignmentResult.synchronize();
					} catch (CancellationException e) {
						context.getFutureResult(0).cancel(true);
					} catch (ExecutionException e) {
						context.getFutureResult(0).cancel(true);
					} catch (InterruptedException e) {
						context.getFutureResult(0).cancel(true);
					}

					return new AlignmentFrame((AlignmentTree) traceAlignmentResult.getResult(0), activityColorMap);
				}
			}
		}
		context.getFutureResult(0).cancel(true);
		return null;
	}

	static class ChildLogger implements Logger {

		public void log(String message, PluginContextID contextID, MessageLevel messageLevel) {
			// ignore
		}

		public void log(Throwable t, PluginContextID contextID) {
			// ignore
		}

	}

}
